#include <netinet/tcp.h>
#include <netinet/udp.h>

#include "libpktmatch.h"

/* Linked list of modifications. */
struct pkm_syn_modifications
{
	struct pkm_syn_modifications *next;

	u_int32_t modification_start;
	u_int32_t modification_end;

	/* Modification hangs off end. */
	unsigned char mod[0];
};

/* TCP: we keep a map of packets previously sent. */
struct pkm_internal_tcp
{
	u_int32_t seq_start_of_data;
	u_int32_t seq_end_of_data;
	unsigned char *data;
	unsigned char *mask;

	struct pkm_syn_modifications *mods;
};

struct pkm_internal 
{
	const struct pkm_pattern *pattern;
	struct pkm_internal_tcp tcp_info;
};

#define NEW(type, contents...) 				\
({ 							\
	typeof (type *) _n = malloc(sizeof(type)); 	\
	if (_n) *_n = ( (type) ## contents);		\
	_n;						\
})

static size_t max_pattern_length(const struct pkm_pattern *pattern)
{
	size_t total = 0;

	while (pattern) {
		total += pattern->maximum_repeats;
		pattern = pattern->next;
	}

	return total;
}

/* Use the first packet in that direction to init. */
pkm_handle
pkm_handle_init_tcp(const struct pkm_pattern *pattern,
		    const struct tcphdr *tcph)
{
	size_t i, maxlen = max_pattern_length(pattern);
	unsigned char *data_and_mask = malloc(maxlen * 2);
	struct pkm_internal *handle;

	if (!data_and_mask)
		return NULL;

	if (!(handle = NEW(struct pkm_internal, 
			   { pattern, { ntohl(tcph->syn), 
					ntohl(tcph->syn), 
					data_and_mask,
					data_and_mask + maxlen, NULL } }))) {
		free(data_and_mask);
		return NULL;
	}

	return handle;
}

pkm_handle
pkm_handle_init_udp(const struct pkm_pattern *pattern,
		    const struct udphdr *udph)
{
	return NEW(struct pkm_internal, {pattern, 
			{ 0, 0, NULL, NULL, NULL }});
}

static int try_match(const struct pkm_pattern *pattern,
		     const unsigned char *data,
		     size_t datalen)
{
	const struct pkm_pattern *ptr;
	size_t i;

	for (i = 0, ptr=pattern; i < datalen; i++) {
		if (
	

/* Search for pattern in TCP packet; returns one of the above, or
 * position in packet. */
int 
pkm_find_tcp(const struct tcphdr *tcph, size_t pktlen, pkm_handle *handle)
{
	unsigned int i, matchoffset;
	unsigned char *data = (unsigned char *)tcph + (tcph->doff * 4);
	size_t datalen = pktlen - (tcph->doff * 4);

	/* If it could be part of a match, but isn't a complete match,
	   drop it. */
	for (i = 0, matchoffset = 0; i < datalen && 


we can't be sure (we
	don't have all the pieces for the match from offset onwards in
	this packet), we have to drop it. */
	
	

	

 /* Do we have a
	match? */
	

	/* Is the tcp packet off the end of the range?  Drop it. */
	if (ntohl(tcph->seq) > handle->tcp_info.seq_end_of_data)
		return PKM_MUST_DROP;

	/* Merge with the current pattern */
	

/* All TCP packets must pass through this, even if replacement_length is 0:
   gives a chance to adjust the sequence number. */
extern int 
pkm_process_tcp(struct tcphdr *tcph, 
		size_t offset, 
		unsigned char *replacement,
		size_t replacement_length,
		size_t length_to_replace,
		pkm_handle *handle);

/* All TCP packets going the other way must go through this, to adjust
   ack numbers. */
extern int 
pkm_process_tcp_reply(struct tcphdr *tcph, pkm_handle *handle);

/* Always searches from start of packet + offset. */
extern int
pkm_find_udp(const struct udphdr *udph, pkm_handle *handle, size_t offset);
#endif /*_LIBPKTMATCH_H*/
