#ifndef _LIBPKTMATCH_H
#define _LIBPKTMATCH_H
/* My brain hurts. */

/* We want to handle retransmissions, packet loss, everything.  We
   can't.  Consider the following (we want to replace the second part):

	PORT 192,168,1,1,4,1

   This can be sent in many ways, but most common variation (after a
   single packet) is:
	"PORT "
	"192,168,1,1,4,1"

   Here's the trick.  We drop packets which match part of the pattern,
   and rely on retransmit to fix it up.
*/

/* A pattern consists of a linked list of these. */
struct pkm_pattern
{
	struct pkm_pattern *next;
	unsigned int num_possibilities;
	unsigned char *possibilities;
	unsigned int minimum_repeats, maximum_repeats;
};

typedef struct pkm_internal *pkm_handle;

struct tcphdr;
struct udphdr;

/* Use the first SYN/ACK packet to init. */
extern pkm_handle *
pkm_handle_init_tcp(const struct pkm_pattern *pattern,
		    const struct tcphdr *tcph);

extern pkm_handle *
pkm_handle_init_udp(const struct pkm_pattern *pattern,
		    const struct udphdr *udph);

#define PKM_MUST_DROP -2
#define PKM_NOT_FOUND -1

/* Search for pattern in TCP packet; returns one of the above, or
 * position in packet.  Offset specifies the offset into the pattern
 * (in bytes) which you need; the packet must contain from this offset
 * to the end of the pattern.*/
extern int 
pkm_find_tcp(const struct tcphdr *tcph, pkm_handle *handle, size_t offset);

/* All TCP packets must pass through this, even if replacement_length is 0:
   gives a chance to adjust the sequence number. */
extern int 
pkm_process_tcp(struct tcphdr *tcph, 
		size_t offset, 
		unsigned char *replacement,
		size_t replacement_length,
		size_t length_to_replace,
		pkm_handle *handle);

/* All TCP packets going the other way must go through this, to adjust
   ack numbers. */
extern int 
pkm_process_tcp_reply(struct tcphdr *tcph, pkm_handle *handle);

/* Always searches from start of packet + offset. */
extern int
pkm_find_udp(const struct udphdr *udph, pkm_handle *handle, size_t offset);

#endif /*_LIBPKTMATCH_H*/
